// 🐦 Flutter imports:
import 'package:flutter/material.dart';

// 📦 Package imports:
import 'package:fluent_ui/fluent_ui.dart' as fluent;
import 'package:go_router/go_router.dart';
import 'package:provider/provider.dart';
import 'package:provider/single_child_widget.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

typedef ThemedRoute = GoRoute;

typedef ThemedRouter = GoRouter;

class ThemedApp extends StatelessWidget {
  final String title;
  final bool debugShowCheckedModeBanner;
  final bool showPerformanceOverlay;
  final Widget errorPageWidget;
  final List<ThemedRoute> routes;
  final List<SingleChildWidget>? providers;
  final bool setUrlPathStrategy;
  const ThemedApp({
    Key? key,
    required this.title,
    required this.routes,
    this.setUrlPathStrategy = true,
    this.providers,
    this.errorPageWidget = const ThemedMessage(
      message: "404 - Unknown Page",
    ),
    this.debugShowCheckedModeBanner = true,
    this.showPerformanceOverlay = false,
  }) : super(key: key);

  _router() => GoRouter(
        urlPathStrategy: setUrlPathStrategy ? UrlPathStrategy.path : null,
        routes: routes,
        errorPageBuilder: (context, state) => MaterialPage<void>(
          key: state.pageKey,
          child: errorPageWidget,
        ),
      );

  @override
  Widget build(BuildContext context) {
    final router = _router();
    return MultiProvider(
      providers: [
        ChangeNotifierProvider(create: (_) => VertticeUIState()),
        ...?providers,
      ],
      child: Consumer<VertticeUIState>(
        builder: (context, value, _) {
          return Theme(
            data: ThemeData(
              useMaterial3: true,
              primaryColor: value.themedColors.accentColor,
              colorScheme: ColorScheme.fromSeed(
                seedColor: value.themedColors.accentColor,
                brightness:
                    value.isDarkTheme ? Brightness.dark : Brightness.light,
              ),
            ),
            child: fluent.FluentApp.router(
              routeInformationProvider: router.routeInformationProvider,
              routeInformationParser: router.routeInformationParser,
              routerDelegate: router.routerDelegate,
              color: value.themedColors.accentColor.toAccentColor(),
              theme: fluent.ThemeData(
                iconTheme: IconThemeData(
                  size: 24,
                  color: Colors.grey[850],
                ),
                accentColor: value.themedColors.accentColor.toAccentColor(),
                brightness: Brightness.light,
                visualDensity: VisualDensity.comfortable,
              ),
              darkTheme: fluent.ThemeData(
                iconTheme: const IconThemeData(
                  size: 24,
                  color: Colors.white,
                ),
                accentColor: value.themedColors.accentColor.toAccentColor(),
                brightness: Brightness.dark,
                visualDensity: VisualDensity.comfortable,
              ),
              showPerformanceOverlay: showPerformanceOverlay,
              themeMode: value.isDarkTheme ? ThemeMode.dark : ThemeMode.light,
              title: title,
              debugShowCheckedModeBanner: debugShowCheckedModeBanner,
            ),
          );
        },
      ),
    );
  }
}
